////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2016 RWS Inc, All Rights Reserved
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of version 2 of the GNU General Public License as published by
// the Free Software Foundation
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
//
// localize.cpp
// Project: Postal
//
// This module deals with localization issues.
//
// History:
//		11/19/96 MJR	Started.
//
//		01/21/97	JMI	Added g_pszAssetsMissingError for when the condition occurs
//							that is an error when the assets are unfound by the nostril.
//
//		01/22/97	JMI	Added g_pszSaveFileQuery.
//
//		01/22/97 MJR	Added g_pszBadCDPath, g_pszBadHDPath, g_pszBadVDPath
//
//		02/03/97	JMI	Added g_pszBadNoSakDirPath[].
//
//		04/11/97	JMI	Added g_pszSaveDemoTitle and g_pszGeneralWriteError.
//
//		04/16/97 MJR	Added a few more messages and cleaned up text for others.
//
//							And then added a few more video messages, too.
//
//		04/21/97 MJR	Created generic version of "bad path" message.
//
//		05/14/97	JMI	Added g_pszPickedUpMessage_ld_s for CDude.
//
//		06/03/97	JMI	Changed g_pszAudioModeError to g_pszAudioModeGeneralError
//							and added g_pszAudioModeInUseError,
//							g_pszAudioModeNoDeviceError,
//							g_pszAudioModeNotSupportedError.
//
//		06/04/97	JMI	Added g_pszNotOnCDROM.
//
//		06/06/97	JMI	Changed message for g_pszPickedUpMessage_ld_s and its name
//							to g_pszPickedUpMessage_s.
//
//		06/14/97 MJR	Added/enhanced general file error messages.
//
//		07/13/97	JMI	Changed g_pszAudioModeNotSupportedError to
//							g_pszAudioModeNotSupportedError_s and changed to ask the
//							user if they want to try the vanilla mode.
//							Also, changed the other audio errors to include the
//							sprintf format parameters in their names.
//							Also, added g_pszAudioVanillaModeNotSupportedError_s for
//							the case when the hardware does not support the vanilla
//							mode.
//
//		07/18/97 BRH	Added strings for new dialogs for loading and saving
//							games.
//
//		07/21/97	JMI	Added g_pszNoWeaponButHaveAmmo_s and g_pszNoWeapon_s.
//
//		07/28/97	JMI	Added g_pszDispenserNoDispenseeTypeChosen.
//
//    07/30/97	BRH	Added death messages that come up when the CDude dies.
//
//		08/05/97	JMI	Added g_pszDontHaveExecuteWeapon_s and
//							g_pszDontHaveSuicideWeapon_s.
//							Also, made g_sLocalizeNumDeadMessages get its value based
//							on the number of elements in g_apszDeathMessages[] so we
//							don't have to worry about keeping that up to date.
//
//		08/12/97	JMI	Added g_pszGenericBrowseFor_s_Title and
//							g_pszGenericMustBeRelativePath_s.
//
//		08/17/97	JMI	Got rid of m_szMessages and all message related functions
//							and variables from CDude since we are now using the toolbar
//							for dude status feedback to the user.  This includes:
//							MsgTypeInfo, m_lNextStatusUpdateTime, m_lMsgUpdateDoneTime,
//							m_print, m_bClearedStatus, m_szMessages[], m_sDeadMsgNum,
//							ms_amtfMessages[], ms_u8FontForeIndex, ms_u8FontBackIndex,
//							ms_u8FontShadowIndex, DrawStatus(), StatusChange(),
//							MessageChange(), Message(), UpdateFontColors(),
//							CPowerUp::ms_apszPowerUpTypeNames[],
//							CPowerUp::GetDescription(), and some strings and a string
//							array in localize.*.
//
//		08/20/97	JMI	Added g_pszDontDropYourselfMORON.
//
//		08/21/97	JMI	Added g_pszDoofusCannotFindNavNet_EditMode_hu_hu and
//							g_pszDoofusCannotFindNavNet_PlayMode_hu_hu.
//
//		08/25/97	JMI	Added g_pszCannotOpenSoundFiles_s_s.
//
//		08/27/97	JMI	Upgraded g_pszCannotOpenSoundFiles_s_s message.
//
//		09/09/97 MJR	Changed g_pszBadBlueInit to include reference to DirectX.
//
//					MJR	Further changed g_pszBadBlueInit to include NT info.
//
//		09/11/97	JMI	Added g_pszPlayOneRealmOnlyMessage.
//
//		09/18/97	JMI	Added localization vars for menus.
//
//		09/18/97	JMI	Added localization vars for SampleMaster categories.
//
//		09/19/97	JMI	Columnized menu strings so it would be easier to copy
//							and paste from here into Excel and back.
//
//		09/23/97	JMI	Added localizable sections for menus for US/UK, FRENCH, &
//							GERMAN.
//
//		09/24/97 BRH	Made ini file based on LOCALE.
//
//		09/26/97	JMI	Added French & German for Menus.
//
//		09/29/97	JMI	Added g_pszNoSoundFiles.
//
//		09/30/97	JMI	g_pszCantFindAssets was missing the word 'in' in the
//							clause "you may need to change the  '%s =' entry the file
//							PREFS_FILE to specify where the APP_NAME files are
//							located" between 'entry' and 'the file'.
//
//		09/30/97 BRH	Changed postalUK.ini to postalEU.ini since it applies
//							to all of the foreign versions.
//
//		10/07/97	JMI	Changed PostalEU.ini to PostalUK.INI, PostalFr.INI, and
//							PostalGr.INI.
//
//		10/07/97 BRH	Added Score localizations to the file.
//
//		10/09/97	JMI	Added g_pszVideoChangeDepthErrorUnderGDI_s and modified
//							g_pszVideoChangeDepthError to only mention the part about
//							Windows' help under Win32.
//
//		10/13/97	JMI	Added g_pszControlsMenu_UseJoystick to localizable texts.
//
//		12/04/97 BRH	Added g_pszStartSinglePlayerMenu_AddOn.
//
//		03/05/98 BRH	Added g_pszWrongCD message that tells the player that they
//							must have the original Postal CD in the drive in order to
//							play the Postal Add on Pack.
//
//		09/27/99	JMI	Added conditions for LOCALE == JAPAN.
//
//		10/07/99	JMI	Added conditions for TARGET == JAPAN_ADD_ON and
//							TARGET == SUPER_POSTAL.
//
//		11/28/99 MJR	For JAPAN_ADD_ON changed the .ini file to "Postal.ini" so
//							that the installation process is easier, since it can then
//							just use the same .ini as the original version of Postal.
//
//		12/02/99 MJR	Changed JAPAN_ADD_ON back to using a special .ini file.
//
//		02/04/00 MJR	Added g_pszPromptForOriginalCD.
//
//		03/30/00 MJR	Moved APP_NAME and PREFS_FILE macro definitions into
//							CompileOptions.h.
//							Switched to using new START_MENU_ADDON_ITEM macro to
//							control whether there's an add-on item on the START menu.
//							Added new POSTAL_PLUS text.
//
//		04/02/00 MJR	Major changes to lots of error messages.  The idea was to
//							reduce the verbage and instead refer the user to the
//							help files for details.
//
//		04/03/00 MJR	New text for POSTAL_PLUS start menu items.
//
//		06/25/01	MJR	Added some new messages.
//
//		05/07/03 MJR	Changed error messages that said "CD-ROM drive" to say
//						"drive it was installed from".  This should help clear
//						up confusion for people that have multiple drives such
//						as DVD, CD-R, etc.
//
////////////////////////////////////////////////////////////////////////////////
#define LOCALIZE_CPP

#include "RSPiX.h"
#include "localize.h"
#include "CompileOptions.h"
#include "realm.h"

////////////////////////////////////////////////////////////////////////////////
// Macros/types/etc.
////////////////////////////////////////////////////////////////////////////////

// Determines the number of elements in the passed array at compile time.
#define NUM_ELEMENTS(a) (sizeof(a) / sizeof(a[0]))

////////////////////////////////////////////////////////////////////////////////
// Variables/data
////////////////////////////////////////////////////////////////////////////////

extern char g_pszAppName[] = APP_NAME;
extern char g_pszCriticalErrorTitle[] = APP_NAME;
extern char g_pszPrefFileName[] = PREFS_FILE;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////// Menus ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern char *g_pszMainMenu_Title[NUM_LANGUAGES] 					= {MAIN_MENU_TITLE, MAIN_MENU_TITLE, MAIN_MENU_TITLE, MAIN_MENU_TITLE, MAIN_MENU_TITLE, MAIN_MENU_TITLE};
extern char *g_pszMainMenu_Start[NUM_LANGUAGES] 					= {"START", "SPIEL", "DEMARRER", "INICIO", "STARTEN", "INIZIO"};
extern char *g_pszMainMenu_Options[NUM_LANGUAGES] 					= {"OPTIONS", "OPTIONEN", "OPTIONS", "OPCIONES", "OPTIES", "OPZIONI"};
extern char *g_pszMainMenu_Credits[NUM_LANGUAGES] 					= {"CREDITS", "CREDITS", "CREDITS", "CREDITOS", "AFTITELING", "CREDITI"}; // todo: accent CRÉDITOS
extern char *g_pszVerifyQuitMenu_Title[NUM_LANGUAGES]				= {"REALLY QUIT?",  "WIRKLICH BEENDEN?", "VRAIMENT QUITTER ?", "QUIERES ABANDONAR?", "ECHT STOPPEN?", "VUOI USCIRE?"}; // todo: ¿QUIERES ABANDONAR?
extern char *g_pszVerifyQuitMenu_Yes[NUM_LANGUAGES] 				= {"YES", "JA", "OUI", "SI", "JA", "SI"};
extern char *g_pszVerifyQuitMenu_No[NUM_LANGUAGES] 					= {"NO", "NEIN", "NON", "NO", "NEE", "NO"};
extern char *g_pszGameMenu_Title[NUM_LANGUAGES] 					= {"GAME", "SPIEL", "JEU", "PARTIDA", "SPELEN", "PARTITA"};
extern char *g_pszGameMenu_Continue[NUM_LANGUAGES] 					= {"CONTINUE", "WEITER", "CONTINUER", "CONTINUAR", "DOORGAAN", "CONTINUA"};
extern char *g_pszGameMenu_Options[NUM_LANGUAGES] 					= {"OPTIONS", "OPTIONEN", "OPTIONS", "OPCIONES", "OPTIES", "OPZIONI"};
extern char *g_pszGameMenu_Quit[NUM_LANGUAGES]						= {"QUIT", "BEENDEN", "QUITTER", "SALIR", "STOPPEN", "ESCI"};
extern char *g_pszGameMenu_Suicide[NUM_LANGUAGES]					= {"SUICIDE", "SELBSTMORD", "SUICIDE", "SUICIDIO", "ZELFMOORD", "SUICIDIO"};
extern char *g_pszStartGameMenu_GameStart[NUM_LANGUAGES] 			= {"GAME START", "SPIEL STARTEN", "COMMENCER JEU", "COMENZAR PARTIDA", "SPEL STARTEN", "INIZIA PARTITA"};
extern char *g_pszOptionsMenu_Audio[NUM_LANGUAGES]					= {"AUDIO", "AUDIO", "AUDIO", "SONIDO", "GELUID", "AUDIO"};
extern char *g_pszOptionsMenu_Controls[NUM_LANGUAGES]				= {"CONTROLS", "STEUERUNG", "COMMANDES", "CONTROLES", "BEDIENING", "CONTROLLI"};
extern char *g_pszDifficultyMenu_Title[NUM_LANGUAGES] 				= {"DIFFICULTY", "SCHWIERIGKEIT", "DIFFICULTE", "DIFICULTAD", "MOEILIJKHEID", "DIFFICOLTA"};
extern char *g_pszAudioMenu_Title[NUM_LANGUAGES]					= {"AUDIO", "AUDIO", "AUDIO", "SONIDO", "GELUID", "AUDIO"};
extern char *g_pszAudioMenu_Mixer[NUM_LANGUAGES] 					= {"MIXER", "MISCHPULT", "MIXER", "MEZCLADOR", "MIXER", "MIXER"};
extern char *g_pszAudioMenu_Language[NUM_LANGUAGES] 				= {"LANGUAGE", "SPRACHE", "LANGUE", "IDIOMA", "TAAL", "LINGUA"};
extern char *g_pszAudioMenu_PainFrequency[NUM_LANGUAGES] 			= {"PAIN FREQ.", "SCHMERZ FREQ.", "FREQ. DOULER", "FRECUENCIA DE DOLOR", "PIJN FREQUENTIE", "FREQUENZA DOLORE"};
extern char *g_pszAudioMenu_English[NUM_LANGUAGES]					= {"English", "Englisch", "Anglais", "Ingles", "Engels", "Inglese"}; // todo accent Inglés
extern char *g_pszAudioMenu_Japanese[NUM_LANGUAGES] 				= {"Japanese", "Japanisch", "Japonais", "Japones", "Japans", "Giapponese"}; // todo accent Japonés
extern char *g_pszVideoMenu_Title[NUM_LANGUAGES] 					= {"VIDEO", "VIDEO", "VIDEO", "VIDEO", "VIDEO", "VIDEO"}; // todo accent on VÍDEO
extern char *g_pszControlsMenu_Title[NUM_LANGUAGES] 				= {"CONTROLS", "STEUERUNG", "COMMANDES", "CONTROLES", "BEDIENING", "CONTROLLI"};
extern char *g_pszRotationSetupMenu_RestoreDefaults[NUM_LANGUAGES] 	= {"RESTORE DEFAULTS", "EINSTELLUNGEN ZURUECKSETZEN", "DEFAUT", "RESTABLECER VALORES POR DEFECTO", "HERSTEL INSTELLINGEN", "VALORI PREDEFINITI"}; // todo really long
extern char *g_pszVolumesMenu_Title[NUM_LANGUAGES] 					= {"AUDIO MIXER", "AUDIO-MISCHPULT", "MIXER AUDIO", "MEZCLADOR DE SONIDO", "GELUIDSMIXER", "MIX AUDIO"};
extern char *g_pszStartGameMenu_Title[NUM_LANGUAGES] 				= {"START GAME", "SPIEL STARTEN", "DEMARRE LE JEU", "COMENZAR PARTIDA", "SPEL STARTEN", "INIZIA PARTITA"};
extern char *g_pszGameOptionsMenu_Easy[NUM_LANGUAGES] 				= {"EASY", "KINDERSPIEL", "FACILE", "FACIL", "MAKKELIJK", "FACILE"}; // todo: accent FÁCIL
extern char *g_pszGameOptionsMenu_Medium[NUM_LANGUAGES] 			= {"NORMAL", "NORMAL", "NORMAL", "NORMAL", "NORMAAL", "NORMALE"};
extern char *g_pszGameOptionsMenu_Hard[NUM_LANGUAGES] 				= {"HARD", "HARD", "DIFFICILE", "DIFICIL", "MOEILIJK", "DIFFICILE"}; // todo: accent DIFÍCIL
extern char *g_pszGameOptionsMenu_Nightmare[NUM_LANGUAGES] 			= {"LABYRINTH ZONE", "LABYRINTH-ZONE", "CAUCHEMAR", "PESADILLA", "NACHTMERRIE", "INCUBO"};
extern char *g_pszDifficulty_Easy[NUM_LANGUAGES] 					= {"Easy", "Kinderspiel", "Facile", "Facil", "Makkelijk", "Facile"}; // todo: accent FÁCIL
extern char *g_pszDifficulty_Medium[NUM_LANGUAGES] 					= {"Normal", "Normal", "Normal", "Normal", "Normaal", "Normale"};
extern char *g_pszDifficulty_Hard[NUM_LANGUAGES] 					= {"Hard", "Hard", "Difficile", "Dificil", "Moeilijk", "Difficile"}; // todo: accent DIFÍCIL
extern char *g_pszDifficulty_Nightmare[NUM_LANGUAGES] 				= {"L-Zone", "L-Zone", "Cauchemar", "Pesadilla", "Nachtmerrie", "Incubo"};
extern char *g_pszStartSinglePlayerMenu_NewCampaign[NUM_LANGUAGES] 	= {"NEW CAMPAIGN", "NEUE KAMPAGNE", "NOUVELLE CAMPAGNE", "NUEVA CAMPANA", "NIEUWE CAMPAGNE", "NUOVA CAMPAGNA"}; // todo: accent CAMPAÑA
extern char *g_pszStartSinglePlayerMenu_New[NUM_LANGUAGES] 			= {"POSTAL", "POSTAL", "CHARGER ORIGINAL", "POSTAL", "POSTAL", "POSTAL"};
extern char *g_pszStartSinglePlayerMenu_AddOn[NUM_LANGUAGES] 		= {"SPECIAL DELIVERY", "SPECIAL DELIVERY DLC", "CHARGER ADD-ON", "ENTREGA ESPECIAL", "SPECIALE LEVERING", "SPECIAL DELIVERY"};
extern char *g_pszStartSinglePlayerMenu_AddOn2[NUM_LANGUAGES] 		= {"SUPER POSTAL",  "SUPER POSTAL",  "SUPER POSTAL", "SUPER POSTAL", "SUPER POSTAL", "SUPER POSTAL"};
extern char *g_pszStartSinglePlayerMenu_AllLevels[NUM_LANGUAGES] 	= {"EXCESS POSTAGE (ALL LEVELS)", "ALLE LEVEL", "TOUS LES NIVEAUX", "TODOS LOS NIVELES", "ALLE LEVELS", "EXCESS POSTAGE (TUTTI I LIVELLI)"};
extern char *g_pszStartSinglePlayerMenu_LoadLevel[NUM_LANGUAGES] 	= {"LEVEL SELECT", "LEVEL SELECT", "CHARGER LE NIVEAU", "SELECCION DE NIVEL", "LEVEL SELECTEREN", "SELEZIONA LIVELLO"};
extern char *g_pszStartSinglePlayerMenu_LoadGame[NUM_LANGUAGES] 	= {"LOAD GAME", "SPIEL LADEN", "CHARGER LE JEU", "CARGAR PARTIDA", "SPEL LADEN", "CARICA GIOCO"};
extern char *g_pszStartSinglePlayerMenu_Challenge[NUM_LANGUAGES] 	= {"CHALLENGE", "HERAUSFORDERUNGEN", "DEFI", "DESAFIO", "UITDAGING", "SFIDA"}; //todo: accent DESAFÍO
extern char *g_pszStartSinglePlayerMenu_Coop[NUM_LANGUAGES] 		= {"CO-OP CAMPAIGN", "KO-OP KAMPAGNE", "CAMPAGNE COOPERATIVE", "CAMPANA COOPERATIVA", "CO-OP CAMPAGNE", "CAMPAGNA COOPERATIVA"}; // todo: accent CAMPAÑA
extern char *g_pszStartSinglePlayerMenu_ChangeSave[NUM_LANGUAGES] 	= {"CHANGE SAVE FILE", "SPIELSTAND AUSWAEHLEN", "CHANGER FICHIER SAUVEGARDE", "CAMBIAR PARTIDA GUARDADA", "OPGESLAGEN SPEL WIJZIGEN", "CAMBIA SALVATAGGIO FILE"};
extern char *g_pszStartChallengeMenu_Gauntlet[NUM_LANGUAGES] 		= {"THE GAUNTLET", "DER SPIESSRUTENLAUF", "LE GANT", "EL DESAFIO", "SPITSROEDEN LOPEN", "FORCHE CAUDINE"}; // todo: accent DESAFÍO
extern char *g_pszStartChallengeMenu_Timed[NUM_LANGUAGES]			= {"TIMED", "ZEITGESTEUERT", "CHRONOMETRE", "TIEMPO", "GETIMED", "TEMPO"};
extern char *g_pszStartChallengeMenu_Goal[NUM_LANGUAGES] 			= {"GOAL", "ZIEL", "BUT", "META", "DOEL", "OBBIETTIVO"};
extern char *g_pszStartChallengeMenu_Flag[NUM_LANGUAGES] 			= {"FLAG", "FLAGGE", "DRAPEAU", "BANDERA", "VLAG", "BANDIERA"};
extern char *g_pszStartChallengeMenu_CheckPoint[NUM_LANGUAGES] 		= {"CHECKPOINT", "CHECKPOINT", "CONTROLE", "CONTROL", "CONTROLEPUNT", "CHECKPOINT"};
extern char *g_pszMultiplayerSetupMenu_Color[NUM_LANGUAGES] 		= {"COAT COLOR", "MANTEL FARBE", "COULEUR", "COLOR", "KLEUR JAS", "COLORE"};
extern char *g_pszInputHold[NUM_LANGUAGES] 							= {"(HOLD)", "(HALTE)", "(MAINTENIR)", "(MANTENER)", "(HOUVAST)", ""}; // Todo: TIENI PREMUTO (IT) too long
extern char *g_pszInputDuck[NUM_LANGUAGES] 							= {"DUCK", "DUCKE", "S'ACCROUPIR", "AGACHARSE", "BUKKEN", "ABBASSARSI"};
extern char *g_pszInputFire[NUM_LANGUAGES] 							= {"SHOOT", "SCHIESS", "TIRER", "DISPARAR", "SCHIETEN", "SPARARE"};
extern char *g_pszInputExecute[NUM_LANGUAGES]						= {"EXECUTE", "EXEKUTIERE", "EXECUTER", "EJECUTAR", "UITVOEREN", "ESEGUIRE"};
extern char *g_pszInputRevive[NUM_LANGUAGES] 						= {"REVIVE", "WIEDERBELEBEN", "RESSUCITER", "REVIVIR", "NIEUW LEVEN", "RISORGERE"};
extern char *g_pszInputCoop[NUM_LANGUAGES] 							= {"CO-OP", "KO-OP", "COOPERATIF", "COOPERACION", "CO-OP", "CO-OP"}; // todo: accent COOPERACIÓN
extern char *g_pszInputLockOn[NUM_LANGUAGES] 						= {"LOCK-ON", "LOCK-ON", "VERROUILAGE", "BLOQUEAR", "VERGRENDELEN\nOP DOEL", "BLOCCA\nSUL BERSAGLIO"};
extern char *g_pszInputSuicide[NUM_LANGUAGES] 						= {"SUICIDE", "SELBSTMORD", "SUICIDE", "SUICIDIO", "ZELFMOORD", "SUICIDIO"};
extern char *g_pszInputAimOnSpot[NUM_LANGUAGES] 					= {"AIM-ON-SPOT", "IM STAND\nZIELEN", "VISER SUR PLACE", "APUNTAR\nSIN MOVERSE", "TER\nPLAATSE\nRICHTEN", "PUNTA\nSENZA\n     MUOVERE"};
extern char *g_pszInputMove[NUM_LANGUAGES] 							= {"MOVE", "BEWEGE", "MOUVEMENT", "MOVIMIENTO", "BEWEEG", "MUOVI"};
extern char *g_pszInputAim[NUM_LANGUAGES] 							= {"AIM", "ZIELEN", "VISER", "APUNTAR", "RICHTEN", "PUNTAMENTO"};

extern char *g_pszInputMoveUp[NUM_LANGUAGES] 						= {"MOVE UP", "NACH\nOBEN BEWEGEN", "ALLER EN HAUT", "ARRIBA", "OMHOOG", "SU"};
extern char *g_pszInputMoveRight[NUM_LANGUAGES] 					= {"MOVE RIGHT", "NACH\nRECHTS\nBEWEGEN", "ALLER A\nDROITE", "DERECHA", "RECHTS", "DESTRA"};
extern char *g_pszInputMoveDown[NUM_LANGUAGES] 						= {"MOVE DOWN", "NACH\nUNTEN\nBEWEGEN", "ALLER EN BAS", "ABAJO", "OMLAAG", "GIU"};
extern char *g_pszInputMoveLeft[NUM_LANGUAGES] 						= {"MOVE LEFT", "NACH\nLINKS\nBEWEGEN", "ALLER A GAUCHE", "IZQUIERDA", "LINKS", "SINISTRA"};

extern char *g_pszInputAimUp[NUM_LANGUAGES] 						= {"AIM UP", "NACH\nOBEN ZIELEN", "VISER EN HAUT", "APUNTAR\nARRIBA", "OMHOOG RICHTEN", "PUNTA SU"};
extern char *g_pszInputAimRight[NUM_LANGUAGES] 						= {"AIM RIGHT", "NACH\nRECHTS\nZIELEN", "VISER A\nDROITE", "APUNTAR\nDERECHA", "RECHTS\nRICHTEN", "PUNTA\nDESTRA"};
extern char *g_pszInputAimDown[NUM_LANGUAGES] 						= {"AIM DOWN", "NACH\nUNTEN\nZIELEN", "VISER EN BAS", "APUNTAR\nABAJO", "OMLAAG\nRICHTEN", "PUNTA GIU"};
extern char *g_pszInputAimLeft[NUM_LANGUAGES] 						= {"AIM LEFT", "NACH\nLINKS ZIELEN", "VISER A GAUCHE", "APUNTAR IZQUIERDA", "LINKS\nRICHTEN", "PUNTA SINISTRA"};

extern char *g_pszInputMenuConfirm[NUM_LANGUAGES] 					= {"MENU CONFIRM", "MENUE BESTAETIGEN", "CONFIRMER CHOIX MENU", "CONFIRMAR ELECCION DE MENU", "MENU BEVESTIGEN", "CONFERMA SCELTA MENU"}; // todo: accent ELECCIÓN | MENÚ
extern char *g_pszInputMenuBack[NUM_LANGUAGES] 						= {"MENU BACK", "MENUE ZURUECK", "RETOUR ARRIERE", "VOLVER AL MENU", "MENU TERUG", "MENU INDIETRO"}; // todo: accent MENÚ
extern char *g_pszInputPause[NUM_LANGUAGES] 						= {"PAUSE", "PAUSE", "PAUSE", "PAUSA", "PAUSE", "PAUSA"};
extern char *g_pszInputPrev[NUM_LANGUAGES] 							= {"PREV.", "ZURUECK", "PREC.", "PREVIO", "VORIGE", "PRECEDENTE"};
extern char *g_pszInputNext[NUM_LANGUAGES] 							= {"NEXT", "WEITER", "SUIVANT", "SIGUIENTE", "VOLGENDE", "PROSSIMO"};
extern char *g_pszInputWeapon[NUM_LANGUAGES] 						= {"WEAPON", "WAFFE", "ARMES", "ARMAS", "WAPEN", "ARMA"};
extern char *g_pszInputGrenade[NUM_LANGUAGES] 						= {"GRENADE", "GRANATE", "GRENADE", "GRANADA", "GRANAAT", "GRANATA"};
extern char *g_pszInputMolotov[NUM_LANGUAGES] 						= {"MOLOTOV", "MOLOTOV", "MOLOTOV", "COCTEL MOLOTOV", "MOLOTOV", "MOLOTOV"}; // todo: accent CÓCTEL MÓLOTOV
extern char *g_pszInputAssisted[NUM_LANGUAGES] 						= {"ASSISTED (A)", "UNTERSTUETZT (A)", "ASSISTE (A)", "ASISTIDO (A)", "GEASSISTEERD (A)", "ASSISTITO (A)"};
extern char *g_pszInputAssistedAlt[NUM_LANGUAGES] 					= {"ASSISTED (B)", "UNTERSTUETZT (B)", "ASSISTE (B)", "ASISTIDO (B)", "GEASSISTEERD (B)", "ASSISTITO (B)"};
extern char *g_pszInputWingman[NUM_LANGUAGES] 						= {"WINGMAN", "WINGMAN", "WINGMAN", "WINGMAN", "WINGMAN", "WINGMAN"};
extern char *g_pszInputFineAim[NUM_LANGUAGES] 						= {"FINE-AIM", "FEINES ZIELEN", "VISEE PRECISE", "MEJORAR PUNTERIA", "ACCURAAT RICHTEN", "MIRA DI PRECISIONE"}; // todo: accent "MEJORAR PUNTERÍA"
extern char *g_pszInputTwinStick[NUM_LANGUAGES] 					= {"TWIN-STICK", "TWIN-STICK", "TWIN STICK", "TWIN-STICK", "TWIN-STICK", "TWIN-STICK"};
extern char *g_pszLanguage[NUM_LANGUAGES] 							= {"LANGUAGE", "SPRACHE", "LANGUE", "IDIOMA", "TAAL", "LINGUA"};
extern char *g_pszMenuLanguages[NUM_LANGUAGES] 						= {"ENGLISH", "DEUTSCH", "FRANCAIS", "ESPANOL", "NEDERLANDS", "ITALIANO"}; // todo: accent ESPAÑOL
extern char *g_pszLanguages[NUM_LANGUAGES] 							= {"English", "Deutsch", "Francais", "Espanol", "Nederlands", "Italiano"}; // todo: accent Español
extern char *g_pszInterface[NUM_LANGUAGES] 							= {"INTERFACE", "INTERFACE", "INTERFACE", "INTERFAZ", "INTERFACE", "INTERFACCIA"};
extern char *g_pszPlayer[NUM_LANGUAGES] 							= {"PLAYER", "SPIELER", "JOUEUR", "JUGADOR", "SPELER", "GIOCATORE"};
extern char *g_pszController[NUM_LANGUAGES]							= {"CONTROLLER", "CONTROLLER", "MANETTE", "CONTROL", "CONTROLLER", "CONTROLLER"};
extern char *g_pszPort[NUM_LANGUAGES]								= {"PORT", "PORT", "PORT", "PUERTO", "POORT", "PORTA"};
extern char *g_pszSaving[NUM_LANGUAGES] 							= {"SAVING...", "SPEICHERT...", "SAUVEGARDE EN COURS...", "GUARDADO EN CURSO...", "BEZIG MET OPSLAAN...", "SALVATAGGIO..."};
extern char *g_pszPressStart[NUM_LANGUAGES]							= {"Press Start", "Druecke Start", "Appuyez Sur Start", "Pulsa Start", "Druk Op Start", "Premi Start"};
extern char *g_pszPressStartUc[NUM_LANGUAGES]						= {"PRESS START", "DRUECKE START", "APPUYEZ SUR START", "PULSA START", "DRUK OP START", "PREMI START"};
extern char *g_pszOK[NUM_LANGUAGES]									= {"OK", "OKAY", "OK", "SI", "OKE", "OK"};
extern char *g_pszReboot[NUM_LANGUAGES]								= {"Reboot", "Neustart", "REINITIALISER", "REINICIAR", "Herstart", "Riavvia"};
extern char *g_pszLoadSave[NUM_LANGUAGES]							= {"LOAD SAVE FILE", "LADE SPIELSTAND", "CHARGER FICHIER SAUVEGARDE", "CARGAR PARTIDA GUARDADA", "OPGESLAGEN SPEL LADEN", "CARICA SALVATAGGIO GIOCO"};
extern char *g_pszExitL[NUM_LANGUAGES]								= {"< EXIT", "< BEENDEN", "< SORTIR", "< SALIR", "< STOPPEN", "< ESCI"};
extern char *g_pszExitR[NUM_LANGUAGES] 								= {"EXIT >", "BEENDEN >", "SORTIR >", "SALIR >", "STOPPEN >", "ESCI >"};
extern char *g_pszKills[NUM_LANGUAGES]								= {"Kills", "Ermordungen", "Victimes", "Victimas", "Slachtoffers", "Vittime"}; // Todo: accent  Víctimas
extern char *g_pszDeaths[NUM_LANGUAGES]								= {"Deaths", "Tode", "Morts", "Muertes", "Doden", "Morti"};
extern char *g_pszRevives[NUM_LANGUAGES]							= {"Revives", "Wiederbelebungen", "Ressucitations", "Resurrecciones", "Aantal nieuwe levens", "Risorti"};
extern char *g_psz_Off[NUM_LANGUAGES] 								= {"Off", "Aus", "Desactive", "Apagado", "Uit", "Spento"};

extern char *g_pszNoMemoryCard[NUM_LANGUAGES] = 
{
	"MEMORY CARD MISSING",
	"MEMORY KARTE FEHLT",
	"CARTE MEMOIRE MANQUANTE",
	"FALTA TARJETA DE MEMORIA",
	"GEHEUGENKAART MIST",
	"MEMORY CARD ASSENTE"
};

extern char *g_pszChooseMemoryCard[NUM_LANGUAGES] = 
{
	"CHOOSE MEMORY CARD",
	"WAEHLE MEMORY KARTE",
	"CHOISIR CARTE MEMOIRE",
	"ELEGIR TARJETA DE MEMORIA",
	"GEHEUGENKAART KIEZEN",
	"SCEGLI MEMORY CARD"
};

extern char *g_pszCoopCampaign[NUM_LANGUAGES] = 
{
	"CHOOSE CAMPAIGN",
	"WAEHLE EINE KAMPAGNE",
	"CHOISIR CAMPAGNE",
	"ELEGIR CAMPANA", // todo: accent CAMPAÑA
	"CAMPAGNE KIEZEN",
	"SCEGLI CAMPAGNA"
};

extern char *g_pszGameNoSave[NUM_LANGUAGES][2] =
{
	{"Game progress will", "not be saved!"},	
	{"Der Spielstand wird", "NICHT gespeichert!"},
	{"Ta progression ne", "sera pas sauvegardee !"}, // todo: accent sauvegardée
	{"El progreso de la ", "partida no se guardara!"}, // todo: upside down ¡El, accent guardará
	{"Voortgang wordt ", "niet opgeslagen!"},
	{"I progressi del gioco", "non verranno salvati!"}	 
};

extern char *g_pszGameNoSaveFound[NUM_LANGUAGES] = 
{
	"NO SAVE FILE FOUND",
	"KEIN SPIELSTAND GEFUNDEN",
	"PAS DE SAUVEGARDE TROUVEE",
	"NO SE HA ENCONTRADO PARTIDA GUARDADA",
	"GEEN OPGESLAGEN SPEL GEVONDEN",
	"NESSUN SALVATAGGIO TROVATO"
};

extern char *g_pszGameNewSave[NUM_LANGUAGES] = 
{
	"Create A New Save File",
	"Erstelle Einen Neuen Spielstand",
	"Creer Une Nouvelle Sauvegarde",
	"Crear Una Nueva Partida Guardada",
	"Nieuw Opgeslagen Spel Maken",
	"Crea Un Nuovo Salvataggio"
};

extern char *g_pszGameNoSpace[NUM_LANGUAGES][3] =
{   
	{"No save file found and", "not enough free blocks", "to create one"},
	{"Kein Spielstand gefunden und", "nicht genug freie Bloecke", "um einen zu erstellen"},
	{"Pas de fichier de sauvegarde", "trouve et pas assez de blocs", "pour en creer un."}, // todo: accent trouvé | créer
	{"No se ha encontrado partida", "guardada y no hay suficientes", "bloques libres para crear una."},
	{"Geen opgeslagen spel gevonden", "en niet genoeg opslag om er een", "te maken."},
	{"Nessun salvataggio trovato e", "non ci sono abbastanza blocchi", "liberi per crearne uno nuovo."}
};

extern char *g_pszRecheckMemoryCard[NUM_LANGUAGES] = {
	"Re-Check Memory Card",
	"Ueberpruefe Die Memory Karte",	
	"Re-Verifier Carte Memoire",
	"Volver A Comprobar Tarjeta De Memoria",
	"Geheugenkaart Nakijken",
	"Ricontrolla Memory Card"
};

extern char *g_pszChooseAnotherVmu[NUM_LANGUAGES] = 
{
	"Try Another Memory Card",
	"Versuche eine andere Memory Karte",
	"Essayer Une Autre Carte Memoire", // todo: accent mémoire
	"Prueba Otra Tarjeta De Memoria",
	"Probeer Een Andere Geheugenkaart",
	"Prova Un'altra Memory Card"
};

extern char *g_pszVmuWriteError[NUM_LANGUAGES] = 
{
	"Error Saving Game",
	"Fehler beim speichern",
	"Erreur De Sauvegarde",
	"Error al guardar la partida",
	"Fout bij het opslaan",
	"Errore nel salvataggio"
};

extern char *g_pszVmuRemoved[NUM_LANGUAGES] = 
{
	"Memory card removed",
	"Memory Karte entfernt",
	"Carte memoire retiree",	// todo: accent mémoire retirée
	"Tarjeta de memoria extraida", // todo: accent extraída
	"Geheugenkaart verwijderd",
	"Memory card rimossa"
};

extern char *g_pszContinueNoSave[NUM_LANGUAGES] = 
{
	"Continue Without Saving",
	"Weiter Ohne Speichern",
	"Continuer Sans Sauvegarde",
	"Continua Sin Guardar",	// todo: accent Continúa
	"Doorgaan Zonder Op Te Slaan",
	"Continua Senza Salvare"
};

extern char *g_pszNewSave[NUM_LANGUAGES] = 
{
	"NEW SAVE FILE",
	"NEUER SPIELSTAND",
	"NOUVEAU FICHIER DE SAUVEGARDE",
	"NUEVO ARCHIVO DE GUARDADO",
	"NIEUW OPGESLAGEN SPEL MAKEN",
	"NUOVO SALVATAGGIO"
};


extern char *g_pszDeviceRemoved[NUM_LANGUAGES] = 
{
	"DEVICE DISCONNECTED",
	"VERBINDUNG GETRENNT",
	"PERIPHERIQUE DECONNECTE",
	"DISPOSITIVO DESCONECTADO",
	"APPARAAT LOSGEKOPPELD",
	"DISPOSITIVO SCOLLEGATO"
};

extern char *g_pszReconnectController[NUM_LANGUAGES] = 
{
	"Re-connect controller",
	"Schliesse den Controller wieder an",
	"Reconnecter la manette",
	"Volver a conectar el mando",
	"Verbind controller opnieuw",
	"Riconnetti controller"
};

extern char *g_pszReconnectVmu[NUM_LANGUAGES] = 
{
	"Re-connect memory card",
	"Schliesse die Memory Karte wieder an",	
	"Reconnecter la carte memoire",
	"Volver a conectar la tarjeta de memoria",
	"Geheugenkaart opnieuw aansluiten",
	"Riconnetti memory card"
};

extern char *g_pszInterfaces[2][NUM_LANGUAGES] = {
	{"Classic", "Klassisch", "Classique", "Clasico", "Klassiek", "Classica"}, // todo: accent Clásico
	{"Minimal", "Minimal", "Minimal", "Minimo", "Minimaal", "Minimale"}, // todo: accent Mínimo	
};

extern char *g_pszMemoryCardFault[NUM_LANGUAGES] = 
{
	"Memory card fault",
	"Memory Karten fehler",
	"Erreur carte memoire",
	"Fallo en tarjeta de memoria",
	"Geheugenkaart fout",
	"Errore Memory Card"
};

extern char *g_pszInputAssistedTips[NUM_LANGUAGES] = {
	"TIPS: HOLD DOWN THE B-BUTTON TO AIM ON THE SPOT, OR A TO LOCK-ON TO A TARGET WHEN THE BUTTON-PROMPT APPEARS.",
	"TIPP: HALTE DIE B TASTE GEDRUECKT UM AUF DER STELLE ZU BLEIBEN UND ZU ZIELEN, ODER HALTE A GEDRUECKT UM DICH AUF EINEN FEIND ZU FIXIEREN WEN DIE EINGABEAUFFORDERUNG AUFBLINKT.",
	"ASTUCE : MAINTIENS LE BOUTON B POUR VISER SUR PLACE, OU A POUR VERROUILLER UNE CIBLE QUAND LA COMMANDE APPARAIT.",
	"CONSEJOS: MANTEN PULSADO EL BOTON B PARA APUNTAR EN EL ACTO, O EL A, PARA FIJAR UN OBJETIVO CUANDO APAREZCA LA INDICACION DEL BOTON.", // todo accent: CONSEJOS: MANTÉN PULSADO EL BOTÓN B PARA APUNTAR EN EL ACTO, O EL A, PARA FIJAR UN OBJETIVO CUANDO APAREZCA LA INDICACIÓN DEL BOTÓN
	"TIPS: HOUD DE B-KNOP INGEDRUKT OM TER PLAATSE TE RICHTEN, OF A OM OP EEN DOELWIT TE VERGRENDELEN WANNEER DE KNOP-PROMPT VERSCHIJNT.",
	"SUGGERIMENTI: TENERE PREMUTO IL PULSANTE B PER MIRARE OPPURE A PER AGGANCIARE UN BERSAGLIO QUANDO APPARE IL PROMPT DEL PULSANTE."
};
extern char *g_pszInputAssistedAltTips[NUM_LANGUAGES] = {
	"TIPS: ALTERNATIVE MAPPING FOR THE 'ASSISTED (A)' CONTROL-SET, WHERE THE DUCK AND LOCK-ON ACTION BUTTONS ARE SWAPPED.",
	"TIPP: ALTERNATIVE FUER DIE 'ASSISTED (A)' CONTROLLER KONFIGURATION IN WELCHER DIE ZIEL UND DUCK TASTE GETAUSCHT SIND.",
	"ASTUCE : CONFIGURATION ALTERNATIVE POUR LE MODE DE CONTROLE 'ASSISTE (A)', DANS LEQUEL LE BOUTON POUR S'ACCROUPIR ET CELUI DE VERROUILLAGE SONT INVERSES.",
	"CONSEJOS: MAPEO ALTERNATIVO PARA EL CONJUNTO DE CONTROLES \"ASISTIDOS (A)\", DONDE SE INTERCAMBIAN LOS BOTONES DE ACCION DE AGACHARSE Y DE BLOQUEO.", // todo: accent "CONSEJOS: MAPEO ALTERNATIVO PARA EL CONJUNTO DE CONTROLES \"ASISTIDOS (A)\", DONDE SE INTERCAMBIAN LOS BOTONES DE ACCIÓN DE AGACHARSE Y DE BLOQUEO."
	"TIPS: ALTERNATIEVE MAPPING VOOR DE 'ASSISTED (A)' CONTROL-SET, WAAR DE DUCK EN LOCK-ON ACTION KNOPPEN WORDEN VERWISSELD.",
	"SUGGERIMENTI: MAPPATURA ALTERNATIVA PER IL SET DI CONTROLLO 'ASSISTITO (A)', DOVE I PULSANTI ABBASSARSI E AGGANCIA BERSAGLIO SONO SCAMBIATI"
};
extern char *g_pszInputWingmanTips[NUM_LANGUAGES] = {
	"TIPS: ABXY-BUTTONS MIMIC A SECOND ANALOG-STICK FOR TWIN-STICK AIMING. EXECUTE/REVIVE AND SUICIDE ACTIONS ARE MAPPED TO THE D-PAD. THIS MODE HAS A MODERATE AIM-ASSIST.",
	"TIPP: DIE ABXY TASTEN AHMEN EINEN ZWEITEN ANALOG STICK NACH FUER TWIN-STICK ZIELEN. HINRICHTUNGS/WIEDERBELBUNGS UND SELBSTMORD TASTEN SIND IN DIESEN MODUS AUF DAS STEUERKREUZ GELEGT. DIESER MODUS HAT MAESSIGE ZIELHILFE.",
	"ASTUCE : LES BOUTONS ABXY SONT UTILISES EN TANT QUE SECOND STICK ANALOGIQUE POUR UN GAMEPLAY TWIN STICK. LES ACTIONS EXECUTER/RESSUCITER ET SUICIDE SONT UTILISES SUR LA CROIX DIRECTIONNELLE. CE MODE A UNE ASSISTANCE DE VISEE MODEREE.",
	"CONSEJOS: LOS BOTONES ABXY IMITAN UN SEGUNDO STICK ANALOGICO PARA APUNTAR CON UN DOBLE STICK. LAS ACCIONES DE EJECUCION/REANIMACION Y SUICIDIO SE ASIGNAN AL BOTON DIRECCIONAL. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERIA MODERADA.", //todo: accent CONSEJOS: LOS BOTONES ABXY IMITAN UN SEGUNDO STICK ANALÓGICO PARA APUNTAR CON UN DOBLE STICK. LAS ACCIONES DE EJECUCIÓN/REANIMACIÓN Y SUICIDIO SE ASIGNAN AL BOTÓN DIRECCIONAL. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERÍA MODERADA.
	"TIPS: ABXY-KNOPPEN KUNNEN EEN TWEEDE ANALOGE STICK NA VOOR HET RICHTEN MET TWIN-STICK. UITVOEREN/HERVOEREN EN ZELFMOORDACTIES WORDEN TOEGEPAST AAN DE D-PAD. DEZE MODUS HEEFT EEN MATIGE AIM-ASSIST.",
	"SUGGERIMENTI: I PULSANTI ABXY FUNGONO DA SECONDO ANALOG STICK PER LA MIRA CON IL TWINSTICK. LE AZIONI DI ESEGUI/RISORGERE E SUICIDIO SONO MAPPATE SUL D-PAD. QUESTA MODALITÀ HA UN MODERATO AIUTO DI MIRA."
};
extern char *g_pszInputFineAimTips[NUM_LANGUAGES] = {
	"TIPS: 360-DEGREE AIMING MODE WHERE MOVEMENT IS MAPPED TO THE ABXY-BUTTONS AND AIMING IS MAPPED TO THE ANALOG-STICK. THIS MODE HAS A MILD AIM-ASSIST",
	"TIPP: 360-GRAD ZIEL MODUS WO BEWEGUNG MIT DEN ABXY TASTEN GESTEUERT WERDEN UND DAS ZIELEN MIT DEM ANALOG STICK: DIESER MODUS HAT MINIMALE ZIEL HILFE.",
	"ASTUCE : MODE DE VISEE A 360 DEGRES DANS LEQUEL LES BOUTONS ABXY SONT UTILISES POUR LE MOUVEMENT ET LE STICK ANALOGIQUE  EST UTILISE POUR VISER. CE MODE A UNE ASSISTANCE DE VISEE MODEREE",
	"CONSEJOS: MODO DE PUNTERIA DE 360 GRADOS DONDE EL MOVIMIENTO SE ASIGNA A LOS BOTONES ABXY Y LA PUNTERIA SE ASIGNA AL BOTON ANALOGICO. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERIA LEVE.", // todo: accent CONSEJOS: MODO DE PUNTERÍA DE 360 GRADOS DONDE EL MOVIMIENTO SE ASIGNA A LOS BOTONES ABXY Y LA PUNTERÍA SE ASIGNA AL BOTÓN ANALÓGICO. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERÍA LEVE.
	"TIPS: 360-GRADEN RICHTMODUS WAAR BEWEGING WORDT TOEGEPAST AAN DE ABXY-KNOPPEN EN HET RICHTEN WORDT TOEGEPAST AAN DE ANALOGE-STICK. DEZE MODUS HEEFT EEN MILD AIM-ASSIST.",
	"SUGGERIMENTI: MODALITA` DI MIRA A 360 GRADI DOVE IL MOVIMENTO E` MAPPATO AI PULSANTI ABXY E LA MIRA E` MAPPATA ALL'ANALOG STICK. QUESTA MODALITA` HA UN LEGGERO AIUTO DI MIRA." // Todo: Accent MODALITÀ
};
extern char *g_pszInputTwinStickTips[NUM_LANGUAGES] = {
	"TIPS: EXECUTE/REVIVE ACTIONS ARE MAPPED TO THE D(PAUSE) BUTTON AND SUICIDE IS MAPPED TO THE IN-GAME MENU OPTION. THIS MODE HAS A MODERATE AIM-ASSIST.",
	"TIPP: HINRICHTUNGS/WIEDERBELEBUNGS TASTEN SIND AUF DIE D TASTE GELEGT (PAUSE) UND DER SELBSTMORD KNOPF IST IM IN-GAME MENUE ZU FINDEN. DIESER MODUS HAT EINE MAESSIGE ZIELHILFE.",
	"ASTUCE : LES ACTIONS EXECUTER/RESSUCITER SONT ACTIVEES PAR LE BOUTON D (PAUSE) ET LE SUICIDE EST ACTIVEE PAR LE BOUTON LE MENU EN JEU. CE MODE A UNE ASSISTANCE DE VISEE MODEREE.",
	"CONSEJOS: LAS ACCIONES DE EJECUTAR/REVIVIR SE ASIGNAN AL BOTON D(PAUSA) Y EL SUICIDIO SE ASIGNA A LA OPCION DE MENU DEL JUEGO. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERIA MODERADA.", // todo: accent CONSEJOS: LAS ACCIONES DE EJECUTAR/REVIVIR SE ASIGNAN AL BOTÓN D(PAUSA) Y EL SUICIDIO SE ASIGNA A LA OPCIÓN DE MENÚ DEL JUEGO. ESTE MODO TIENE UNA ASISTENCIA DE PUNTERÍA MODERADA.
	"TIPS: ACTIES UITVOEREN/HERVOEREN WORDEN TOEGEPAST AAN DE D(PAUZE)-KNOP EN ZELFMOORD WORDT TOEGEPAST AAN DE IN-GAME MENU-OPTIE. DEZE MODUS HEEFT EEN MATIGE AIM-ASSIST.",
	"SUGGERIMENTI: LE AZIONI ESEGUI/RESUSCITA SONO MAPPATE AL PULSANTE D(PAUSA) E IL SUICIDIO E` MAPPATO ALL'OPZIONE DEL MENU DI GIOCO. QUESTA MODALITA` HA UN MODERATO AIUTO DI MIRA." // Todo: Accent MODALITÀ
};

////////////////////////////////////////////////////////////////////////////////
///////////////////////////// Sample Master ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
extern char *g_apszSoundCategories[10][NUM_LANGUAGES] =
{
	{"GENERAL", "GESAMTLAUTSTAERKE", "GENERAL", "GENERAL", "ALGEMEEN","GENERALE"},
	{"SOUNDTRACK", "SOUNDTRACK", "PISTE MUSICALE", "BANDA SONORA", "SOUNDTRACK","COLONNA SONORA"},
	{"WEAPONS", "WAFFEN", "ARMES", "ARMAS", "WAPENS","ARMI"},
	{"FEEDBACK", "FEEDBACK", "FEED-BACK", "COMENTARIO", "FEEEDBACK","FEEDBACK"},
	{"DESTRUCTION", "ZERSTOERUNG", "DESTRUCTION", "DESTRUCCION",	"VERWOESTING","DISTRUZIONE"}, // todo: accent DESTRUCCIÓN
	{"AMBIENT", "UMGEBUNG", "AMBIANTE", "AMBIENTE", "OMGEVINGSGELUID","AMBIENTE"},
	{"DEMON", "DAEMONEN", "DEMON", "DEMONIO", "DUIVEL","DEMONE"},
	{"SHOUTING", "GESCHREI", "CRIS", "GRITOS", "SCHREEUWEN","URLA"},
	{"PAIN", "SCHMERZ", "DOULEUR", "DOLOR", "PIJN","DOLORE"},
	{"SUFFERING", "LEIDEN", "SOUFFRANCE", "SUFRIMIENTO", "LEIDEN","SOFFERENZA"}
};

////////////////////////////////////////////////////////////////////////////////
// Score module
////////////////////////////////////////////////////////////////////////////////
extern char *g_apszScoreDisplayText[CRealm::TotalScoringModes-1][NUM_LANGUAGES] =
	{
		{
			"      Population %d                   Hostiles %d     Killed %d (%d%% / %d%%)", 	// Standard
			"      Bewohner %d                     Feinde %d      Getoetete %d (%d%% / %d%%)",			
			"      Population %d                   Ennemis %d      Tues %d (%d%% / %d%%)", 		// todo: accent Tués
			"      Poblacion %d                    Enemigos %d     Victimas %d (%d%% / %d%%)", 	// todo: accent Población | Víctimas
			"      Bevolking %d                    Vijandigen %d   Gedood %d (%d%% / %d%%)"	,
			"      Popolazione %d                  Nemici %d       Uccisi %d (%d%% / %d%%)"		
		},
		{
			" Time Remaining %d:%2.2d                                Kills %d",					// Timed
			" Verbleibende Zeit %d:%2.2d                             Toetungen %d",			// Timed
			" Temps restant %d:%2.2d                                 Victimes %d",		 		// Timed
			" Tiempo Transcurrido %d:%2.2d                           Muertes %d",		 	// Timed
			" Tijd resterend %d:%2.2d                                Slachtoffers %d",		 	// Timed			
			" Tempo restante %d:%2.2d                                Vittime %d",		 	// Timed	
		},
		{
			" Time Remaining %d:%2.2d            Kills %d               Remaining %d / %d",		// TimedGoal
			" Verbleibende Zeit %d:%2.2d         Toetungen %d            Verbleibend %d",			
			" Temps restant %d:%2.2d             Victimes %d            Restant %d",	 		
			" Tiempo Transcurrido %d:%2.2d       Muertes %d             Restante %d",	
			" Tijd resterend %d:%2.2d            Slachtoffers %d        Resterend %d",
			" Tempo restante %d:%2.2d            Vittime %d             Rimasti %d"
		},
		{
			" Time Remaining %d:%2.2d",															// TimedFlag
			" Verbleibende Zeit %d:%2.2d",														
			" Temps restant %d:%2.2d",													 		
			" Tiempo Transcurrido %d:%2.2d",
			" Tijd resterend %d:%2.2d",
			" Tempo restante %d:%2.2d"
		},
		{
			" Kills %d                     Remaining %d            Time Elapsed %d:%2.2d",		// Goal
			" Toetungen %d                  Verbleibend %d          Verstrichene Zeit %d:%2.2d", 		
			" Victimes %d                  Restant %d              Temps ecoule %d:%2.2d",	 		// todo: accent Temps écoulé
			" Muertes %d                   Restante %d             Tiempo Transcurrido %d:%2.2d", 		
			" Slachtoffers %d              Resterend %d            Tijd verstreken %d:%2.2d",
			" Vittime %d                   Rimasti %d              Tempo trascorso %d:%2.2d"
		},
		{
			" Time Elapsed %d:%2.2d",															// CaptureFlag
			" Verstrichene Zeit %d:%2.2d",														
			" Temps ecoule %d:%2.2d",													 		// todo: accent Temps écoulé
			" Tiempo Transcurrido %d:%2.2d",
			" Tijd verstreken %d:%2.2d",
			" Tempo trascorso %d:%2.2d"
		},
		{
			" Clock %d:%2.2d                  You have %d flags              Flags Remaining %d",						// Checkpoint
			" Uhr %d:%2.2d                 Du hast %d Flaggen           Verbleibende Flaggen %d",	
			" Horloge %d:%2.2d             Tu as %d drapeaux              Drapeaux restants %d",							
			" Reloj %d:%2.2d               Tienes %d banderas             Banderas Restantes %d",
			" Klok %d:%2.2d                Je hebt nog %d vlaggen         Resterende vlaggen %d",
			" Orologio %d:%2.2d               Tu hai %d Bandiere             Bandiere Rimaste %d",
		},
		{
			"Time Remaining %d:%2.2d",															// MPTimed
			"Verbleibende Zeit %d:%2.2d",														// MPTimed
			"Temps restant %d:%2.2d",													 		// MPTimed
			"Tiempo Transcurrido %d:%2.2d",
			"Tijd resterend %d:%2.2d",
			"Tempo restante %d:%2.2d"
		},
		{
			"Time Remaining %d:%2.2d",															// MPFrag
			"Verbleibende Zeit %d:%2.2d",														// MPFrag
			"Temps restant %d:%2.2d",															// MPFrag
			"Tiempo Transcurrido %d:%2.2d",
			"Tijd resterend %d:%2.2d",
			"Tempo restante %d:%2.2d"
		},
		{"", "", "", "", "", ""},																			// MPLastMan,
		{"", "", "", "", "", ""},																			// MPCaptureFlag
		{"", "", "", "", "", ""},																			// MPTimedFlag,
		{"", "", "", "", "", ""},																			// MPTimedFrag
		{"", "", "", "", "", ""},																			// MPLastManFrag
		{"", "", "", "", "", ""},																			// MPLastManTimed
		{"", "", "", "", "", ""},																			// MPLastManTimedFrag
};

extern char *g_apszScoreDisplayTextMin[CRealm::TotalScoringModes-1][NUM_LANGUAGES] =
	{
		{
			"    Population %d    Hostiles %d    Killed %d (%d%% / %d%%)", 	// Standard
			"    Bewohner %d     Feinde %d    Getoetete %d (%d%% / %d%%)",			
			"    Population %d    Ennemis %d    Tues %d (%d%% / %d%%)", 		// todo: accent Tués
			"    Poblacion %d    Enemigos %d    Victimas %d (%d%% / %d%%)", 	// todo: accent Población | Víctimas
			"    Bevolking %d    Vijandigen %d    Gedood %d (%d%% / %d%%)"	,
			"    Popolazione %d    Nemici %d    Uccisi %d (%d%% / %d%%)"		
		},
		{
			"    Time Remaining %d:%2.2d        Kills %d",					// Timed
			"    Verbleibende Zeit %d:%2.2d        Toetungen %d",			// Timed
			"    Temps restant %d:%2.2d        Victimes %d",		 		// Timed
			"    Tiempo Transcurrido %d:%2.2d        Muertes %d",		 	// Timed
			"    Tijd resterend %d:%2.2d        Slachtoffers %d",		 	// Timed			
			"    Tempo restante %d:%2.2d        Vittime %d",		 	// Timed	
		},
		{
			"    Time Remaining %d:%2.2d    Kills %d    Remaining %d / %d",		// TimedGoal
			"    Verbleibende Zeit %d:%2.2d    Toetungen %d    Verbleibend %d",			
			"    Temps restant %d:%2.2d    Victimes %d    Restant %d",	 		
			"    Tiempo Transcurrido %d:%2.2d    Muertes %d    Restante %d",	
			"    Tijd resterend %d:%2.2d    Slachtoffers %d    Resterend %d",
			"    Tempo restante %d:%2.2d    Vittime %d    Rimasti %d"
		},
		{
			"    Time Remaining %d:%2.2d",															// TimedFlag
			"    Verbleibende Zeit %d:%2.2d",														
			"    Temps restant %d:%2.2d",													 		
			"    Tiempo Transcurrido %d:%2.2d",
			"    Tijd resterend %d:%2.2d",
			"    Tempo restante %d:%2.2d"
		},
		{
			"    Kills %d    Remaining %d    Time Elapsed %d:%2.2d",		// Goal
			"    Toetungen %d   Verbleibend %d   Verstrichene Zeit %d:%2.2d", 		
			"    Victimes %d    Restant %d    Temps ecoule %d:%2.2d",	 		// todo: accent Temps écoulé
			"    Muertes %d  Restante %d  Tiempo Transcurrido %d:%2.2d", 		
			"    Slachtoffers %d  Resterend %d  Tijd verstreken %d:%2.2d",
			"    Vittime %d    Rimasti %d    Tempo trascorso %d:%2.2d"
		},
		{
			"    Time Elapsed %d:%2.2d",															// CaptureFlag
			"    Verstrichene Zeit %d:%2.2d",														
			"    Temps ecoule %d:%2.2d",													 		// todo: accent Temps écoulé
			"    Tiempo Transcurrido %d:%2.2d",
			"    Tijd verstreken %d:%2.2d",
			"    Tempo trascorso %d:%2.2d"
		},
		{
			"    Clock %d:%2.2d    You have %d flags    Flags Remaining %d",						// Checkpoint
			"    Uhr %d:%2.2d    Du hast %d Flaggen\n☺    Verbleibende Flaggen %d",	
			"    Horloge %d:%2.2d    Tu as %d drapeaux\n☺    Drapeaux restants %d",							
			"    Reloj %d:%2.2d    Tienes %d banderas\n☺    Banderas Restantes %d",
			"    Klok %d:%2.2d    Je hebt nog %d vlaggen\n☺    Resterende vlaggen %d",
			"    Orologio %d:%2.2d    Tu hai %d Bandiere\n☺    Bandiere Rimaste %d",
		},
		{
			"    Time Remaining %d:%2.2d",															// MPTimed
			"    Verbleibende Zeit %d:%2.2d",														// MPTimed
			"    Temps restant %d:%2.2d",													 		// MPTimed
			"    Tiempo Transcurrido %d:%2.2d",
			"    Tijd resterend %d:%2.2d",
			"    Tempo restante %d:%2.2d"
		},
		{
			"    Time Remaining %d:%2.2d",															// MPFrag
			"    Verbleibende Zeit %d:%2.2d",														// MPFrag
			"    Temps restant %d:%2.2d",															// MPFrag
			"    Tiempo Transcurrido %d:%2.2d",
			"    Tijd resterend %d:%2.2d",
			"    Tempo restante %d:%2.2d"
		},
		{"", "", "", "", "", ""},																			// MPLastMan,
		{"", "", "", "", "", ""},																			// MPCaptureFlag
		{"", "", "", "", "", ""},																			// MPTimedFlag,
		{"", "", "", "", "", ""},																			// MPTimedFrag
		{"", "", "", "", "", ""},																			// MPLastManFrag
		{"", "", "", "", "", ""},																			// MPLastManTimed
		{"", "", "", "", "", ""},																			// MPLastManTimedFrag
};

// ScoreGoalText
extern char *g_apszScoreGoalText[CRealm::TotalScoringModes-1][NUM_LANGUAGES] =
	{
		{
			"      You must kill %d%% of the hostiles.",										// Standard
			"      Du musst %d%% deiner Feinde toeten.",											
			"      Tu dois tuer %d%% ennemis.",												
			"      Debes matar un %d%% de enemigos.",
			"      Je moet %d%% van de vijandigen vermoorden.",
			"      Devi uccidere il %d%% degli nemici."
		},
		{
			" Score as many kills as possible in the time remaining.",							// Timed
			" Erziele in der verbleibenden Zeit so viele Toetungen wie moeglich.",			
			" Fais autant de victimes que possible dans le delai restant.",						// todo: accent Fais autant de victimes que possible dans le délai restant.
			" Consigue el mayor numero de muertes en el tiempo que queda.",						// todo: accent número
			" Scoor zoveel mogelijk doden in de overgebleven tijd.",
			" Ottieni il maggior numero di vittime possibili nel tempo rimanente."
		},				
		{
			" Kill everyone before time runs out.",												// TimedGoal
			" Toeten jeden vor dem Ablauf der Zeit.",										
			" Tue tout le monde avant l'expiration du delai.",									// todo: Tue tout le monde avant l'expiration du délai.
			" Mata a todos antes de que se acabe el tiempo.",
			" Vermoord iedereen voordat de tijd uitloopt.",
			" Uccidi tutti prima che scada il tempo."
		},
		{
			" Capture the flag before time runs out.",											// TimedFlag
			" Nehme vor dem Ablauf der Zeit die Flagge ein.",									
			" Capture le drapeau avant l'expiration du delai.",									// todo: Capture le drapeau avant l'expiration du délai.
			" Captura la bandera antes de que se acabe el tiempo.",
			" Verover de vlag voordat de tijd uitloopt.",
			" Cattura la bandiera prima che scada il tempo."
		},		
		{
			" Kill %d People in as little time as possible.",									// Goal
			" Toete so schnell wie moeglich %d Leute.",
			" Tue %d personnes aussi vite que possible.",	
			" Mata a %d personas en el menor tiempo posible.",
			" Vermoord %d mensen in zo min mogelijk tijd.",
			" Uccidi %d Persone nel minor tempo possibile."
		},
		{
			" Capture the flag in as little time as possible.",									// CaptureFlag
			" Nehme so schnell wie moeglich die Flagge ein.",											
			" Capture le drapeau aussi vite que possible.",
			" Captura la bandera en el menor tiempo posible.",
			" Verover de vlag in zo min mogelijk tijd.",
			" Cattura la bandiera nel minor tempo possibile."
		},
		{
			" Grab as many flags as possible before time runs out.",							// Checkpoint
			" Hole dir vor Ablauf der Zeit so viele Flaggen wie moeglich.",							
			" Capture autant de drapeaux que possible avant l'expiration du delai.", 			// todo: " Capture autant de drapeaux que possible avant l'expiration du délai.
			" Coge todas las banderas posibles antes de que se acabe el tiempo.",
			" Pak zoveel mogelijk vlaggen als mogelijk voordat de tijd uitloopt.",
			" Afferra quante piu bandiere possibile prima che scada il tempo."					//todo: accent più
		},
		{
			" The player with the most kills when time expires is the winner",					// MPTimed
			" Der Spieler, dar nach Ablauf der Zeit die meisten Toetungen hat, ist der Sieger",	// MPTimed
			" Le joueur ayant fait le plus de victimes a l'expiration du delai est le gagnant",	// MPTimed
			"",
			"",
			""
		},
		{
			" The first player to get %d kills wins",											// MPFrag
			" Der erste Spieler mit %d Toetungen hat gewonnen",									// MPFrag
			" Le premier joueur faisant %d victimes gagne le jeu.",								// MPFrag
			"",
			"",
			""
		},
		{"", "", "", "", "", ""},																				// MPLastMan,
		{"", "", "", "", "", ""},																		// MPCaptureFlag
		{"", "", "", "", "", ""},																		// MPTimedFlag
		{
			" Try to reach %d kills before time expires",										// MPTimedFrag
			" Versuchen Sie, vor Ablauf der Zeit %d Toetungen zu erreichen",						// MPTimedFrag
			" Essayez de faire %d victimes avant l'expiration du delai",							// MPTimedFrag
			"",
			""
		},
		{"", "", "", "", "", ""},																				// MPLastManFrag
		{"", "", "", "", "", ""},																				// MPLastManTimed
		{
			" There are no time or kill limits on this game - play as long as you like", 							// MPLastManTimedFrag
			" In diesem Spiel gibt es keine Ziet- oder Totungsbeschrakungen - spielen Sie so lange wie Sie wollen", // MPLastManTimedFrag
			" Aucune limite de temps ou de victimes dans ce jeu - jouez aussi longtemps que vous le souhaitez.",	// MPLastManTimedFrag
			"",
			"",
			""
		}
};

// ScoreGoalText
extern char *g_apszScoreGoalTextMin[CRealm::TotalScoringModes-1][NUM_LANGUAGES] =
	{
		{
			"    You must kill %d%% of the hostiles.",										// Standard
			"    Du musst %d%% deiner Feinde toeten.",											
			"    Tu dois tuer %d%% ennemis.",												
			"    Debes matar un %d%% de enemigos.",
			"    Je moet %d%% van de vijandigen vermoorden.",
			"    Devi uccidere il %d%% degli nemici."
		},
		{
			"    Score as many kills as possible in the time\n☺    remaining.",							// Timed
			"    Erziele in der verbleibenden Zeit so\n☺    viele Toetungen wie moeglich.",			
			"    Fais autant de victimes que possible dans\n☺    le delai restant.",						// todo: accent Fais autant de victimes que possible dans le délai restant.
			"    Consigue el mayor numero de muertes en el\n☺    tiempo que queda.",						// todo: accent número
			"    Scoor zoveel mogelijk doden in de\n☺    overgebleven tijd.",
			"    Ottieni il maggior numero di vittime\n☺    possibili nel tempo rimanente."
		},				
		{
			"    Kill everyone before time runs out.",												// TimedGoal
			"    Toeten jeden vor dem Ablauf der Zeit.",										
			"    Tue tout le monde avant l'expiration du delai.",									// todo: Tue tout le monde avant l'expiration du délai.
			"    Mata a todos antes de que se acabe el tiempo.",
			"    Vermoord iedereen voordat de tijd uitloopt.",
			"    Uccidi tutti prima che scada il tempo."
		},
		{
			"    Capture the flag before time runs out.",											// TimedFlag
			"    Nehme vor dem Ablauf der Zeit die Flagge ein.",									
			"    Capture le drapeau avant l'expiration du delai.",									// todo: Capture le drapeau avant l'expiration du délai.
			"    Captura la bandera antes de que se acabe\n☺    el tiempo.",
			"    Verover de vlag voordat de tijd uitloopt.",
			"    Cattura la bandiera prima che scada il tempo."
		},		
		{
			"    Kill %d People in as little time as possible.",									// Goal
			"    Toete so schnell wie moeglich %d Leute.",						    	
			"    Tue %d personnes aussi vite que possible.",	
			"    Mata a %d personas en el menor tiempo posible.",
			"    Vermoord %d mensen in zo min mogelijk tijd.",
			"    Uccidi %d Persone nel minor tempo possibile."
		},
		{
			"    Capture the flag in as little time as possible.",									// CaptureFlag
			"    Nehme so schnell wie moeglich die Flagge ein.",											
			"    Capture le drapeau aussi vite que possible.",
			"    Captura la bandera en el menor tiempo posible.",
			"    Verover de vlag in zo min mogelijk tijd.",
			"    Cattura la bandiera nel minor tempo possibile."
		},
		{
			"    Grab as many flags as possible before time\n☺    runs out.",							// Checkpoint
			"\n☺    Hole dir vor Ablauf der Zeit so viele\n☺    Flaggen wie moeglich.",							
			"\n☺    Capture autant de drapeaux que possible avant\n☺    l'expiration du delai.", 			// todo: " Capture autant de drapeaux que possible avant l'expiration du délai.
			"\n☺    Coge todas las banderas posibles antes\n☺    de que se acabe\n☺     el tiempo.",
			"\n☺    Pak zoveel mogelijk vlaggen als mogelijk\n☺    voordat de tijd uitloopt.",
			"\n☺    Afferra quante piu bandiere possibile\n☺    prima che scada il tempo."					//todo: accent più
		},
		{
			" The player with the most kills when time expires is the winner",					// MPTimed
			" Der Spieler, dar nach Ablauf der Zeit die meisten Toetungen hat, ist der Sieger",	// MPTimed
			" Le joueur ayant fait le plus de victimes a l'expiration du delai est le gagnant",	// MPTimed
			"",
			"",
			""
		},
		{
			" The first player to get %d kills wins",											// MPFrag
			" Der erste Spieler mit %d Toetungen hat gewonnen",									// MPFrag
			" Le premier joueur faisant %d victimes gagne le jeu.",								// MPFrag
			"",
			"",
			""
		},
		{"", "", "", "", "", ""},																				// MPLastMan,
		{"", "", "", "", "", ""},																		// MPCaptureFlag
		{"", "", "", "", "", ""},																		// MPTimedFlag
		{
			" Try to reach %d kills before time expires",										// MPTimedFrag
			" Versuchen Sie, vor Ablauf der Zeit %d Toetungen zu erreichen",						// MPTimedFrag
			" Essayez de faire %d victimes avant l'expiration du delai",							// MPTimedFrag
			"",
			""
		},
		{"", "", "", "", "", ""},																				// MPLastManFrag
		{"", "", "", "", "", ""},																				// MPLastManTimed
		{
			" There are no time or kill limits on this game - play as long as you like", 							// MPLastManTimedFrag
			" In diesem Spiel gibt es keine Ziet- oder Totungsbeschrakungen - spielen Sie so lange wie Sie wollen", // MPLastManTimedFrag
			" Aucune limite de temps ou de victimes dans ce jeu - jouez aussi longtemps que vous le souhaitez.",	// MPLastManTimedFrag
			"",
			"",
			""
		}
};

// Units for the various scoring types.
extern char *g_apszScoreUnits[16][NUM_LANGUAGES] =
	{
		{"", "", "", "", "", ""},		 // Standard.
		
		{
			"Kills", 		// Timed.
			"Toetungen",
			"Victimes",
			"Muertes",
			"Slachtoffers",
			"Vittime"
		},
		{"", "", "", "", "", ""},		 // Timed goal.
		{"", "", "", "", "", ""},		 // Timed flag.
		{"", "", "", "", "", ""},		 // Goal.
		{"", "", "", "", "", ""},		 // Capture the flag.
		{
			"Flags", // Checkpoint.
			"Flaggen",	// Checkpoint.
			"Drapeaux",
			"Banderas",
			"Vlaggen",
			"Bandiere"
		}, // Checkpoint.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"},  // Multiplayer timed.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer Frag limited.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer last man standing.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer capture the flag.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer Timed capture the flag.
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer timed frags.		
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer last man frag limited.		
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer last man with time limit.		
		{"Frags", "Toetungen", "Victimes", "Muertes", "Slachtoffers", "Vittime"}, // Multiplayer last man frag and time limited.
};

// Explanations for the various scoring types.
extern char *g_apszScoreExplanations[17][NUM_LANGUAGES] =
	{
		{"", "", "", "", "", ""},		 // Standard.
		{
			"Most kills in %s",					  		// Timed.
			"Die meisten Toetungen in %s",				// Timed.
			"Maximum de victimes en %s",					// Timed.
			"Mas muertes en %s",				// todo: Más muertes en (x)
			"Meeste doden in %s",
			"Maggior numero di vittime in %s"
		},
		{
			"Best times for killing %d hostiles", 		// Timed goal.
			"Bestzeit beim Toeten von %d Feinden", 	// Timed goal.
			"Meilleurs temps pour tuer %d ennemis",   	// Timed goal.
			"El mejor tiempo matando %d enemigos",
			"Beste tijd voor het doden van %d vijandigen",
			"I tempi migliori uccidendo %d nemici"
		},
		{
			"Best times for capturing the flag",  		// Timed flag.
			"Bestzeiten beim Einnehmen der Flagge", 	// Timed flag.
			"Meilleurs temps de capture des drapeaux", 	// Timed flag.
			"El mejor tiempo capturando la bandera",
			"Beste tijd voor het veroveren van de vlag",
			"I tempi migliori catturando e la bandiera"
		},
		{
			"Best times for killing %d hostiles", 		// Goal.
			"Bestzeit beim Toeten von %d Feinden", 	
			"Meilleurs temps pour tuer %d ennemis",   	
			"El mejor tiempo matando %d enemigos",
			"Beste tijd voor het doden van %d vijandigen",
			"I tempi migliori uccidendo %d nemici"
		},
		{
			"Best times for capturing the flag",  		// Capture the flag.
			"Bestzeiten beim Einnehmen der Flagge", 	// Capture the flag.
			"Meilleurs temps de capture des drapeaux", 	// Capture the flag.
			"El mejor tiempo capturando la bandera",
			"Beste tijd voor het veroveren van de vlag",
			"I tempi migliori catturando e la bandiera"
		},
		{
			"Most flags collected",				  		// Checkpoint.				
			"Meiste gesammelte Flaggen",				// Checkpoint.
			"Nombre maximum de drapeaux amasses",	   	// Checkpoint.
			"Numero maximo de banderas recogidas",	// todo: Número máximo de banderas recogidas
			"Meeste vlaggen verzameld",
			"Numero massimo di bandiere raccolte"
		},
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer timed.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer frag.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer last man standing.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer capture the flag.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer Timed capture the flag.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer timed frags.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer last man frag limited.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer last man with time limit.
		{"Top %d Scores", "Hochste %d Punktzahl",	"%d Meilleurs Scores"},	  // Multiplayer last man frag and time limited.
		{
			"Kills - Deaths - Revives", 
			"Ermordungen - Tode - Wiederbelebungen", 
			"Victimes - Morts - Ressucitations",
			"Victimas - Muertes - Resurrecciones",
			"Slachtoffers - Doden - Aantal nieuwe levens",
			"Vittime - Morti - Risorti"
		} // Campaign coop
};

////////////////////////////////////////////////////////////////////////////////
// EOF
////////////////////////////////////////////////////////////////////////////////
